@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2014 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 #define UNUSED(x) (void) x // silence compiler warnings
 
+using namespace std;
+
 namespace meep {
 
 void fields::set_boundary(boundary_side b,direction d,boundary_condition cond){
@@ -89,18 +91,18 @@ void fields::disconnect_chunks() {
   for (int i=0;i<num_chunks;i++) {
     DOCMP {
       FOR_FIELD_TYPES(f)
-	for (int ip=0;ip<3;++ip)
-	  for (int io=0;io<2;io++) {
-	    delete[] chunks[i]->connections[f][ip][io];
-	    chunks[i]->connections[f][ip][io] = NULL;
-	  }
+      	for (int ip=0;ip<3;++ip)
+      	  for (int io=0;io<2;io++) {
+      	    delete[] chunks[i]->connections[f][ip][io];
+      	    chunks[i]->connections[f][ip][io] = NULL;
+      	  }
     }
     FOR_FIELD_TYPES(f) {
       delete[] chunks[i]->connection_phases[f];
       chunks[i]->connection_phases[f] = NULL;
       for (int ip=0;ip<3;++ip)
-	for (int io=0;io<2;io++)
-	  chunks[i]->num_connections[f][ip][io] = 0;
+      	for (int io=0;io<2;io++)
+      	  chunks[i]->num_connections[f][ip][io] = 0;
     }
   }
   FOR_FIELD_TYPES(ft)
@@ -108,7 +110,7 @@ void fields::disconnect_chunks() {
       delete[] comm_blocks[ft][i];
       comm_blocks[ft][i] = 0;
       for (int ip=0;ip<3;++ip)
-	comm_sizes[ft][ip][i] = 0;
+      	comm_sizes[ft][ip][i] = 0;
     }
 }
 
@@ -167,7 +169,7 @@ bool fields::locate_point_in_user_volume(ivec *there, complex<double> *phase) co
 
 void fields::locate_volume_source_in_user_volume(const vec p1, const vec p2, vec newp1[8], vec newp2[8],
                                                   complex<double> kphase[8], int &ncopies) const {
-  // For periodic boundary conditions, 
+  // For periodic boundary conditions,
   // this function locates up to 8 translated copies of the initial grid_volume specified by (p1,p2)
   // First bring center of grid_volume inside
   ncopies = 1;
@@ -175,7 +177,7 @@ void fields::locate_volume_source_in_user_volume(const vec p1, const vec p2, vec
   newp2[0] = p2;
   kphase[0] = 1;
   vec cen = (newp1[0] + newp2[0]) * 0.5;
-  LOOP_OVER_DIRECTIONS(gv.dim, d) 
+  LOOP_OVER_DIRECTIONS(gv.dim, d)
     if (boundaries[High][d] == Periodic)  {
       while (cen.in_direction(d) < gv.boundary_location(Low, d)) {
         newp1[0] += lattice_vector(d);
@@ -190,9 +192,9 @@ void fields::locate_volume_source_in_user_volume(const vec p1, const vec p2, vec
         cen = (newp1[0] + newp2[0]) * 0.5;
       }
     }
-  
+
   // if grid_volume extends outside user_volume in any direction, we need to duplicate already existing copies
-  LOOP_OVER_DIRECTIONS(gv.dim, d) 
+  LOOP_OVER_DIRECTIONS(gv.dim, d)
     if (boundaries[High][d] == Periodic) {
       if (newp1[0].in_direction(d) < gv.boundary_location(Low, d) ||
           newp2[0].in_direction(d) < gv.boundary_location(Low, d)) {
@@ -242,7 +244,7 @@ bool fields::locate_component_point(component *c, ivec *there,
 }
 
 void fields_chunk::zero_metal(field_type ft) {
-  for (int i=0;i<num_zeroes[ft];i++) *(zeroes[ft][i]) = 0.0;
+  for (size_t i=0;i<num_zeroes[ft];i++) *(zeroes[ft][i]) = 0.0;
 }
 
 void fields::find_metals() {
@@ -255,7 +257,7 @@ void fields::find_metals() {
         chunks[i]->num_zeroes[ft] = 0;
         DOCMP FOR_COMPONENTS(c)
           if (type(c) == ft && chunks[i]->f[c][cmp])
-	    LOOP_OVER_VOL_OWNED(vi, c, n) 
+	    LOOP_OVER_VOL_OWNED(vi, c, n)
 	      if (IVEC_LOOP_AT_BOUNDARY) { // todo: just loop over boundaries
 		IVEC_LOOP_ILOC(vi, here);
 		if (on_metal_boundary(here))
@@ -263,15 +265,15 @@ void fields::find_metals() {
 	      }
         typedef realnum *realnum_ptr;
         chunks[i]->zeroes[ft] = new realnum_ptr[chunks[i]->num_zeroes[ft]];
-        int num = 0;
+        size_t num = 0;
         DOCMP FOR_COMPONENTS(c)
           if (type(c) == ft && chunks[i]->f[c][cmp])
-	    LOOP_OVER_VOL_OWNED(vi, c, n)
-	      if (IVEC_LOOP_AT_BOUNDARY) { // todo: just loop over boundaries
-		IVEC_LOOP_ILOC(vi, here);
-		if (on_metal_boundary(here))
-		  chunks[i]->zeroes[ft][num++] = chunks[i]->f[c][cmp] + n;
-	      }
+    	    LOOP_OVER_VOL_OWNED(vi, c, n)
+    	      if (IVEC_LOOP_AT_BOUNDARY) { // todo: just loop over boundaries
+          		IVEC_LOOP_ILOC(vi, here);
+          		if (on_metal_boundary(here))
+          		  chunks[i]->zeroes[ft][num++] = chunks[i]->f[c][cmp] + n;
+    	      }
       }
     }
 }
@@ -283,12 +285,12 @@ bool fields_chunk::needs_W_notowned(component c) {
 }
 
 void fields::connect_the_chunks() {
-  int *nc[NUM_FIELD_TYPES][3][2];
+  size_t *nc[NUM_FIELD_TYPES][3][2];
   FOR_FIELD_TYPES(f)
     for (int ip=0;ip<3;ip++)
       for (int io=0;io<2;io++) {
-	nc[f][ip][io] = new int[num_chunks];
-	for (int i=0;i<num_chunks;i++) nc[f][ip][io][i] = 0;
+      	nc[f][ip][io] = new size_t[num_chunks];
+      	for (int i=0;i<num_chunks;i++) nc[f][ip][io][i] = 0;
       }
 
   /* For some of the chunks, H==B, and we definitely don't need to
@@ -303,7 +305,7 @@ void fields::connect_the_chunks() {
   int *B_redundant = new int[num_chunks*2 * 5];
   for (int i = 0; i < num_chunks; ++i)
     FOR_H_AND_B(hc,bc)
-      B_redundant[5*(num_chunks+i) + bc-Bx] 
+      B_redundant[5*(num_chunks+i) + bc-Bx]
       = chunks[i]->f[hc][0] == chunks[i]->f[bc][0];
   and_to_all(B_redundant + 5*num_chunks, B_redundant, 5*num_chunks);
 
@@ -318,7 +320,7 @@ void fields::connect_the_chunks() {
      array) and a non-PML region (no separate W). */
   bool needs_W_notowned[NUM_FIELD_COMPONENTS];
   FOR_COMPONENTS(c) needs_W_notowned[c] = false;
-  FOR_E_AND_H(c) for (int i=0;i<num_chunks;i++) 
+  FOR_E_AND_H(c) for (int i=0;i<num_chunks;i++)
     needs_W_notowned[c]= needs_W_notowned[c] || chunks[i]->needs_W_notowned(c);
   FOR_E_AND_H(c) needs_W_notowned[c] = or_to_all(needs_W_notowned[c]);
 
@@ -327,71 +329,71 @@ void fields::connect_the_chunks() {
     const grid_volume vi = chunks[i]->gv;
     FOR_FIELD_TYPES(ft)
       for (int ip=0;ip<3;ip++)
-	for (int j=0;j<num_chunks;j++)
-	  comm_sizes[ft][ip][j+i*num_chunks] = 0;
+      	for (int j=0;j<num_chunks;j++)
+      	  comm_sizes[ft][ip][j+i*num_chunks] = 0;
     FOR_COMPONENTS(corig) {
       if (have_component(corig))
-	LOOP_OVER_VOL_NOTOWNED(vi, corig, n) {
-	  IVEC_LOOP_ILOC(vi, here);
-	  component c = corig;
-	  // We're looking at a border element...
-	  complex<double> thephase;
-	  if (locate_component_point(&c,&here,&thephase)
-	      && !on_metal_boundary(here))
-	    for (int j=0;j<num_chunks;j++) {
-	      if ((chunks[i]->is_mine() || chunks[j]->is_mine())
-		  && chunks[j]->gv.owns(here)
-		  && !(is_B(corig) && is_B(c) &&
-		       B_redundant[5*i+corig-Bx] && B_redundant[5*j+c-Bx])) {
-		const int pair = j+i*num_chunks;
-		const connect_phase ip = thephase == 1.0 ? CONNECT_COPY 
-		  : (thephase == -1.0 ? CONNECT_NEGATE : CONNECT_PHASE);
-		{
-		  field_type f = type(c);
-		  const int nn = is_real?1:2;
-		  nc[f][ip][Incoming][i] += nn;
-		  nc[f][ip][Outgoing][j] += nn;
-		  comm_sizes[f][ip][pair] += nn;
-		}
-		if (needs_W_notowned[corig]) {
-		  field_type f = is_electric(corig) ? WE_stuff : WH_stuff;
-		  const int nn = is_real?1:2;
-		  nc[f][ip][Incoming][i] += nn;
-		  nc[f][ip][Outgoing][j] += nn;
-		  comm_sizes[f][ip][pair] += nn;
-		}
-		if (is_electric(corig) || is_magnetic(corig)) {
-		  field_type f = is_electric(corig) ? PE_stuff : PH_stuff;
-		  int ni = 0, cni = 0;
-		  for (polarization_state *pi=chunks[i]->pol[type(corig)]; pi; 
-		       pi = pi->next)
-		    for (polarization_state *pj=chunks[j]->pol[type(c)]; pj;
-			 pj = pj->next)
-		      if (*pi->s == *pj->s) { 
-			if (pi->data && chunks[i]->is_mine()) {
-			  ni += pi->s->num_internal_notowned_needed(corig, 
-								    pi->data);
-			  cni += pi->s->num_cinternal_notowned_needed(corig, 
-								     pi->data);
-			}
-			else if (pj->data && chunks[j]->is_mine()) {
-			  ni += pj->s->num_internal_notowned_needed(c,
-								    pj->data);
-			  cni += pj->s->num_cinternal_notowned_needed(c,
-								    pj->data);
-			}
-		      }
-		  const int nn = (is_real?1:2) * (cni);
-		  nc[f][ip][Incoming][i] += nn;
-		  nc[f][ip][Outgoing][j] += nn;
-		  comm_sizes[f][ip][pair] += nn;
-		  const connect_phase iip = CONNECT_COPY;
-		  nc[f][iip][Incoming][i] += ni;
-		  nc[f][iip][Outgoing][j] += ni;
-		  comm_sizes[f][iip][pair] += ni;
-		}
-	      } // if is_mine and owns...
-	    } // loop over j chunks
+      	LOOP_OVER_VOL_NOTOWNED(vi, corig, n) {
+      	  IVEC_LOOP_ILOC(vi, here);
+      	  component c = corig;
+      	  // We're looking at a border element...
+      	  complex<double> thephase;
+      	  if (locate_component_point(&c,&here,&thephase)
+      	      && !on_metal_boundary(here))
+      	    for (int j=0;j<num_chunks;j++) {
+      	      if ((chunks[i]->is_mine() || chunks[j]->is_mine())
+            		  && chunks[j]->gv.owns(here)
+            		  && !(is_B(corig) && is_B(c) &&
+      		       B_redundant[5*i+corig-Bx] && B_redundant[5*j+c-Bx])) {
+                  const int pair = j+i*num_chunks;
+                  const connect_phase ip = thephase == 1.0 ? CONNECT_COPY
+                    : (thephase == -1.0 ? CONNECT_NEGATE : CONNECT_PHASE);
+                  {
+                    field_type f = type(c);
+                    const int nn = is_real?1:2;
+                    nc[f][ip][Incoming][i] += nn;
+                    nc[f][ip][Outgoing][j] += nn;
+                    comm_sizes[f][ip][pair] += nn;
+                  }
+                  if (needs_W_notowned[corig]) {
+                    field_type f = is_electric(corig) ? WE_stuff : WH_stuff;
+                    const int nn = is_real?1:2;
+                    nc[f][ip][Incoming][i] += nn;
+                    nc[f][ip][Outgoing][j] += nn;
+                    comm_sizes[f][ip][pair] += nn;
+                  }
+                  if (is_electric(corig) || is_magnetic(corig)) {
+                    field_type f = is_electric(corig) ? PE_stuff : PH_stuff;
+                    size_t ni = 0, cni = 0;
+                    for (polarization_state *pi=chunks[i]->pol[type(corig)]; pi;
+                         pi = pi->next)
+                      for (polarization_state *pj=chunks[j]->pol[type(c)]; pj;
+                        	 pj = pj->next)
+                        if (*pi->s == *pj->s) {
+                        	if (pi->data && chunks[i]->is_mine()) {
+                        	  ni += pi->s->num_internal_notowned_needed(corig,
+                        						    pi->data);
+                        	  cni += pi->s->num_cinternal_notowned_needed(corig,
+                        						     pi->data);
+                        	}
+                        	else if (pj->data && chunks[j]->is_mine()) {
+                        	  ni += pj->s->num_internal_notowned_needed(c,
+                        						    pj->data);
+                        	  cni += pj->s->num_cinternal_notowned_needed(c,
+                        						    pj->data);
+                        	}
+                        }
+                    const size_t nn = (is_real?1:2) * (cni);
+                    nc[f][ip][Incoming][i] += nn;
+                    nc[f][ip][Outgoing][j] += nn;
+                    comm_sizes[f][ip][pair] += nn;
+                    const connect_phase iip = CONNECT_COPY;
+                    nc[f][iip][Incoming][i] += ni;
+                    nc[f][iip][Outgoing][j] += ni;
+                    comm_sizes[f][iip][pair] += ni;
+                  }
+                } // if is_mine and owns...
+      	    } // loop over j chunks
         } // LOOP_OVER_VOL_NOTOWNED
     } // FOR_COMPONENTS
 
@@ -411,7 +413,7 @@ void fields::connect_the_chunks() {
      for i < i' */
 
   // wh stores the current indices in the connections array(s)
-  int *wh[NUM_FIELD_TYPES][3][2];
+  size_t *wh[NUM_FIELD_TYPES][3][2];
 
   /* Now allocate the connection arrays... this is still slightly
      wasteful (by a factor of 2) because we allocate for chunks we
@@ -420,142 +422,140 @@ void fields::connect_the_chunks() {
   FOR_FIELD_TYPES(f)
     for (int ip=0;ip<3;ip++) {
       for (int io=0;io<2;io++) {
-	for (int i=0;i<num_chunks;i++)
-	  chunks[i]->alloc_extra_connections(field_type(f),
-					     connect_phase(ip),
-					     in_or_out(io),
-					     nc[f][ip][io][i]);
-	delete[] nc[f][ip][io];
-	wh[f][ip][io] = new int[num_chunks];
+      	for (int i=0;i<num_chunks;i++)
+      	  chunks[i]->alloc_extra_connections(field_type(f),
+      					     connect_phase(ip),
+      					     in_or_out(io),
+      					     nc[f][ip][io][i]);
+      	delete[] nc[f][ip][io];
+      	wh[f][ip][io] = new size_t[num_chunks];
       }
       for (int i=0;i<num_chunks;i++) wh[f][ip][Outgoing][i] = 0;
     }
 
   // Next start setting up the connections...
-  
+
   for (int i=0;i<num_chunks;i++) {
     const grid_volume vi = chunks[i]->gv;
-    
+
     // initialize wh[f][ip][Incoming][j] to sum of comm_sizes for jj < j
     FOR_FIELD_TYPES(f)
       for (int ip=0;ip<3;ip++) {
-	wh[f][ip][Incoming][0] = 0;
-	for (int j = 1; j < num_chunks; ++j)
-	  wh[f][ip][Incoming][j] = wh[f][ip][Incoming][j-1]
-	    + comm_sizes[f][ip][(j-1)+i*num_chunks];
+      	wh[f][ip][Incoming][0] = 0;
+      	for (int j = 1; j < num_chunks; ++j)
+      	  wh[f][ip][Incoming][j] = wh[f][ip][Incoming][j-1]
+      	    + comm_sizes[f][ip][(j-1)+i*num_chunks];
       }
 
     FOR_COMPONENTS(corig)
       if (have_component(corig))
-	LOOP_OVER_VOL_NOTOWNED(vi, corig, n) {
-  	  IVEC_LOOP_ILOC(vi, here);
-	  component c = corig;
-	  // We're looking at a border element...
-	  complex<double> thephase;
-	  if (locate_component_point(&c,&here,&thephase)
-	      && !on_metal_boundary(here))
-	    for (int j=0;j<num_chunks;j++) {
-	      if ((chunks[i]->is_mine() || chunks[j]->is_mine())
-		  && chunks[j]->gv.owns(here)
-		  && !(is_B(corig) && is_B(c) &&
-		       B_redundant[5*i+corig-Bx] && B_redundant[5*j+c-Bx])) {
-		const connect_phase ip = thephase == 1.0 ? CONNECT_COPY 
-		  : (thephase == -1.0 ? CONNECT_NEGATE : CONNECT_PHASE);
-		const int m = chunks[j]->gv.index(c, here);
+      	LOOP_OVER_VOL_NOTOWNED(vi, corig, n) {
+        	  IVEC_LOOP_ILOC(vi, here);
+        	  component c = corig;
+        	  // We're looking at a border element...
+        	  complex<double> thephase;
+        	  if (locate_component_point(&c,&here,&thephase)
+        	      && !on_metal_boundary(here))
+        	    for (int j=0;j<num_chunks;j++) {
+        	      if ((chunks[i]->is_mine() || chunks[j]->is_mine())
+              		  && chunks[j]->gv.owns(here)
+              		  && !(is_B(corig) && is_B(c)
+        		        && B_redundant[5*i+corig-Bx] && B_redundant[5*j+c-Bx])) {
+              		const connect_phase ip = thephase == 1.0 ? CONNECT_COPY
+              		  : (thephase == -1.0 ? CONNECT_NEGATE : CONNECT_PHASE);
+              		const ptrdiff_t m = chunks[j]->gv.index(c, here);
 
-		{
-		  field_type f = type(c);
-		  if (ip == CONNECT_PHASE)
-		    chunks[i]->connection_phases[f][wh[f][ip][Incoming][j]/2] =
-		      thephase;
-		  DOCMP {
-		    chunks[i]->connections[f][ip][Incoming]
-		      [wh[f][ip][Incoming][j]++] = 
-		      chunks[i]->f[corig][cmp] + n;
-		    chunks[j]->connections[f][ip][Outgoing]
-		      [wh[f][ip][Outgoing][j]++] = 
-		      chunks[j]->f[c][cmp] + m;
-		  }
-		}
-		
-		if (needs_W_notowned[corig]) {
-		  field_type f = is_electric(corig) ? WE_stuff : WH_stuff;
-		  if (ip == CONNECT_PHASE)
-		    chunks[i]->connection_phases[f][wh[f][ip][Incoming][j]/2] =
-		      thephase;
-		  DOCMP {
-		    chunks[i]->connections[f][ip][Incoming]
-		      [wh[f][ip][Incoming][j]++] = 
-		      (chunks[i]->f_w[corig][cmp] ? chunks[i]->f_w[corig][cmp]
-		       : chunks[i]->f[corig][cmp]) + n;
-		    chunks[j]->connections[f][ip][Outgoing]
-		      [wh[f][ip][Outgoing][j]++] = 
-		      (chunks[j]->f_w[c][cmp] ? chunks[j]->f_w[c][cmp]
-		       : chunks[j]->f[c][cmp]) + m;
-		  }
-		}
-		
-		if (is_electric(corig) || is_magnetic(corig)) {
-		  field_type f = is_electric(corig) ? PE_stuff : PH_stuff;
-		  for (polarization_state *pi=chunks[i]->pol[type(corig)]; pi; 
-		       pi = pi->next)
-		    for (polarization_state *pj=chunks[j]->pol[type(c)]; pj;
-			 pj = pj->next)
-		      if (*pi->s == *pj->s) {
-			polarization_state *po = NULL;
-			if (pi->data && chunks[i]->is_mine())
-			  po = pi;
-			else if (pj->data && chunks[j]->is_mine())
-			  po = pj;
-			if (po) {
-			  const connect_phase iip = CONNECT_COPY;
-			  const int ni = po->s->
-			    num_internal_notowned_needed(corig, po->data);
-			  for (int k = 0; k < ni; ++k) {
-			    chunks[i]->connections[f][iip][Incoming]
-			      [wh[f][iip][Incoming][j]++] = 
-			      po->s->internal_notowned_ptr(k, corig, n, 
-							   pi->data);
-			    chunks[j]->connections[f][iip][Outgoing]
-			      [wh[f][iip][Outgoing][j]++] =
-			      po->s->internal_notowned_ptr(k, c, m, 
-							   pj->data);
-			  }
-			  const int cni = po->s->
-			    num_cinternal_notowned_needed(corig, po->data);
-			  for (int k = 0; k < cni; ++k) {
-			    if (ip == CONNECT_PHASE)
-			      chunks[i]->connection_phases[f]
-				[wh[f][ip][Incoming][j]/2] = thephase;
-			    DOCMP {
-			      chunks[i]->connections[f][ip][Incoming]
-				[wh[f][ip][Incoming][j]++] =
-				po->s->cinternal_notowned_ptr(k, corig,cmp, n, 
-							      pi->data);
-			      chunks[j]->connections[f][ip][Outgoing]
-				[wh[f][ip][Outgoing][j]++] =
-				po->s->cinternal_notowned_ptr(k, c,cmp, m, 
-							     pj->data);
-			    }
-			  }
-			}
-		      }
-		} // is_electric(corig)
-	      } // if is_mine and owns...
-	    } // loop over j chunks
+              		{
+              		  field_type f = type(c);
+              		  if (ip == CONNECT_PHASE)
+              		    chunks[i]->connection_phases[f][wh[f][ip][Incoming][j]/2] =
+              		      thephase;
+              		  DOCMP {
+              		    chunks[i]->connections[f][ip][Incoming]
+              		      [wh[f][ip][Incoming][j]++] =
+              		      chunks[i]->f[corig][cmp] + n;
+              		    chunks[j]->connections[f][ip][Outgoing]
+              		      [wh[f][ip][Outgoing][j]++] =
+              		      chunks[j]->f[c][cmp] + m;
+              		  }
+              		}
+
+              		if (needs_W_notowned[corig]) {
+              		  field_type f = is_electric(corig) ? WE_stuff : WH_stuff;
+              		  if (ip == CONNECT_PHASE)
+              		    chunks[i]->connection_phases[f][wh[f][ip][Incoming][j]/2] =
+              		      thephase;
+              		  DOCMP {
+              		    chunks[i]->connections[f][ip][Incoming]
+              		      [wh[f][ip][Incoming][j]++] =
+              		      (chunks[i]->f_w[corig][cmp] ? chunks[i]->f_w[corig][cmp]
+              		       : chunks[i]->f[corig][cmp]) + n;
+              		    chunks[j]->connections[f][ip][Outgoing]
+              		      [wh[f][ip][Outgoing][j]++] =
+              		      (chunks[j]->f_w[c][cmp] ? chunks[j]->f_w[c][cmp]
+              		       : chunks[j]->f[c][cmp]) + m;
+              		  }
+              		}
+
+              		if (is_electric(corig) || is_magnetic(corig)) {
+              		  field_type f = is_electric(corig) ? PE_stuff : PH_stuff;
+              		  for (polarization_state *pi=chunks[i]->pol[type(corig)]; pi;
+              		       pi = pi->next)
+              		    for (polarization_state *pj=chunks[j]->pol[type(c)]; pj;
+                    			 pj = pj->next)
+              		      if (*pi->s == *pj->s) {
+                    			polarization_state *po = NULL;
+                    			if (pi->data && chunks[i]->is_mine())
+                    			  po = pi;
+                    			else if (pj->data && chunks[j]->is_mine())
+                    			  po = pj;
+                    			if (po) {
+                    			  const connect_phase iip = CONNECT_COPY;
+                    			  const size_t ni = po->s->
+                    			    num_internal_notowned_needed(corig, po->data);
+                    			  for (size_t k = 0; k < ni; ++k) {
+                    			    chunks[i]->connections[f][iip][Incoming]
+                    			      [wh[f][iip][Incoming][j]++] =
+                    			      po->s->internal_notowned_ptr(k, corig, n,
+                    							   pi->data);
+                    			    chunks[j]->connections[f][iip][Outgoing]
+                    			      [wh[f][iip][Outgoing][j]++] =
+                    			      po->s->internal_notowned_ptr(k, c, m,
+                    							   pj->data);
+                    			  }
+                    			  const size_t cni = po->s->
+                    			    num_cinternal_notowned_needed(corig, po->data);
+                    			  for (size_t k = 0; k < cni; ++k) {
+                    			    if (ip == CONNECT_PHASE)
+                    			      chunks[i]->connection_phases[f]
+                          				[wh[f][ip][Incoming][j]/2] = thephase;
+                      			    DOCMP {
+                      			      chunks[i]->connections[f][ip][Incoming]
+                            				[wh[f][ip][Incoming][j]++] =
+                            				po->s->cinternal_notowned_ptr(k, corig,cmp, n, pi->data);
+                      			      chunks[j]->connections[f][ip][Outgoing]
+                            				[wh[f][ip][Outgoing][j]++] =
+                            				po->s->cinternal_notowned_ptr(k, c,cmp, m, pj->data);
+                      			    }
+                    			  }
+                    			}
+              		      }
+              		} // is_electric(corig)
+        	      } // if is_mine and owns...
+        	    } // loop over j chunks
       } // LOOP_OVER_VOL_NOTOWNED
   } // loop over i chunks
   FOR_FIELD_TYPES(f)
     for (int ip=0;ip<3;ip++)
       for (int io=0;io<2;io++)
-	delete[] wh[f][ip][io];
+      	delete[] wh[f][ip][io];
   delete[] B_redundant;
 }
 
 void fields_chunk::alloc_extra_connections(field_type f, connect_phase ip,
-					   in_or_out io, int num) {
+					   in_or_out io, size_t num) {
   if (num == 0) return; // No need to go to any bother...
-  const int tot = num_connections[f][ip][io] + num;
+  const size_t tot = num_connections[f][ip][io] + num;
   if (io == Incoming && ip == CONNECT_PHASE) {
     delete[] connection_phases[f];
     connection_phases[f] = new complex<realnum>[tot];
